
rm(list = ls())

library(parallel)
library(DEoptim)
library(optimx)
library(mvtnorm)
library(mlogit)

# paper using Cracker data
# Paap and Frances (2000), Journal of Applied Econometrics.
# Jain, Vilcassim and Chintagunta (1994), JBES.
data("Cracker", package = "mlogit")
library(dplyr)
Cracker = filter(Cracker, price.nabisco!=0 & price.sunshine!=0 & 
                   price.keebler!=0 & price.private!=0)
attach(Cracker)

# dependent variable and regressors
NT = nrow(Cracker)
y = seq(0, 0, length = NT)

# normailize price to have mean 0 and std 1
normalize = function(x) {
  log(x/100)
}

id = id
y[choice == "nabisco"] = 1; x11 = normalize(price.nabisco); x12 = disp.nabisco; x13 = feat.nabisco
y[choice == "sunshine"] = 2; x21 = normalize(price.sunshine); x22 = disp.sunshine; x23 = feat.sunshine
y[choice == "keebler"] = 3; x31 = normalize(price.keebler); x32 = disp.keebler; x33 = feat.keebler
y[choice == "private"] = 4; x41 = normalize(price.private); x42 = disp.private; x43 = feat.private

Cracker.data = data.frame(id = id, y = y, p1 = x11, d1 = x12, f1 = x13, p2 = x21, d2 = x22, f2 = x23,
                          p3 = x31, d3 = x32, f3 = x33, p4 = x41, d4 = x42, f4 = x43)
attach(Cracker.data)

# fourth-order Gaussian kernel
ker4 = function(x) {
  (3/2 - 1/2*x^2)*dnorm(x)
}

# pair observations
CN = combn(NT, 2); CombN = split(CN, col(CN))
CombData = function(im) {
  i = im[1]; m = im[2]
  c(id[i], y[i], p1[i], d1[i], f1[i], p2[i], d2[i], f2[i], p3[i], d3[i], f3[i], p4[i], d4[i], f4[i],
    id[m], y[m], p1[m], d1[m], f1[m], p2[m], d2[m], f2[m], p3[m], d3[m], f3[m], p4[m], d4[m], f4[m])
}

dataset = t(sapply(CombN, CombData))

# MRC estimation
MRC.estimation = function(r) {
  
yi = dataset[,2]
xi11 = dataset[,3]; xi12 = dataset[,4]; xi13 = dataset[,5]
xi21 = dataset[,6]; xi22 = dataset[,7]; xi23 = dataset[,8]
xi31 = dataset[,9]; xi32 = dataset[,10]; xi33 = dataset[,11]
xi41 = dataset[,12]; xi42 = dataset[,13]; xi43 = dataset[,14]
  
ym = dataset[,16]
xm11 = dataset[,17]; xm12 = dataset[,18]; xm13 = dataset[,19]
xm21 = dataset[,20]; xm22 = dataset[,21]; xm23 = dataset[,22]
xm31 = dataset[,23]; xm32 = dataset[,24]; xm33 = dataset[,25]
xm41 = dataset[,26]; xm42 = dataset[,27]; xm43 = dataset[,28]
  
yi1 = (yi==1); yi2 = (yi==2); yi3 = (yi==3); yi4 = (yi==4)
ym1 = (ym==1); ym2 = (ym==2); ym3 = (ym==3); ym4 = (ym==4)

a = 5
c1 = sd(xi11-xm11); c2 = sd(xi21-xm21); c3 = sd(xi31-xm31); c4 = sd(xi41-xm41)
hn1 = a*c1*NT^(-1/7); hn2 = a*c2*NT^(-1/7); hn3 = a*c3*NT^(-1/7); hn4 = a*c4*NT^(-1/7)
  
# choice 1, match choice {2,3,4}
K1im = ker4((xi21-xm21)/hn2)*ker4((xi31-xm31)/hn3)*ker4((xi41-xm41)/hn4)*
  (xi22 == xm22)*(xi23 == xm23)*(xi32 == xm32)*(xi33 == xm33)*(xi42 == xm42)*(xi43 == xm43)
R1im = (yi1-ym1) + (ym2-yi2) + (ym3-yi3) + (ym4-yi4)
  
# choice 2, match choice {1,3,4}  
K2im = ker4((xi11-xm11)/hn1)*ker4((xi31-xm31)/hn3)*ker4((xi41-xm41)/hn4)*
  (xi12 == xm12)*(xi13 == xm13)*(xi32 == xm32)*(xi33 == xm33)*(xi42 == xm42)*(xi43 == xm43)
R2im = (ym1-yi1) + (yi2-ym2) + (ym3-yi3) + (ym4-yi4)
  
# choice 3, match choice {1,2,4}
K3im = ker4((xi21-xm21)/hn2)*ker4((xi11-xm11)/hn1)*ker4((xi41-xm41)/hn4)*
  (xi22 == xm22)*(xi23 == xm23)*(xi12 == xm12)*(xi13 == xm13)*(xi42 == xm42)*(xi43 == xm43)
R3im = (ym1-yi1) + (ym2-yi2) + (yi3-ym3) + (ym4-yi4)
  
# choice 4, match choice {1,2,3}
K4im = ker4((xi21-xm21)/hn2)*ker4((xi31-xm31)/hn3)*ker4((xi11-xm11)/hn1)*
  (xi22 == xm22)*(xi23 == xm23)*(xi32 == xm32)*(xi33 == xm33)*(xi12 == xm12)*(xi13 == xm13)
R4im = (ym1-yi1) + (ym2-yi2) + (ym3-yi3) + (yi4-ym4)
  
# MRC objective function using Cracker data
MRCobjN = function(b) {
  Ui1 = (-1)*xi11 + xi12*b[1] + xi13*b[2]; Ui2 = (-1)*xi21 + xi22*b[1] + xi23*b[2]
  Ui3 = (-1)*xi31 + xi32*b[1] + xi33*b[2]; Ui4 = (-1)*xi41 + xi42*b[1] + xi43*b[2]
  Um1 = (-1)*xm11 + xm12*b[1] + xm13*b[2]; Um2 = (-1)*xm21 + xm22*b[1] + xm23*b[2]
  Um3 = (-1)*xm31 + xm32*b[1] + xm33*b[2]; Um4 = (-1)*xm41 + xm42*b[1] + xm43*b[2]
    
  Score = K1im*R1im*sign(Ui1-Um1) + K2im*R2im*sign(Ui2-Um2) + 
          K3im*R3im*sign(Ui3-Um3) + K4im*R4im*sign(Ui4-Um4)
  2*(-1)*sum(Score)
}
  
# DEoptim searching for global optimum 
lb = seq(-1, -1, length = 2); ub = seq(1, 1, length = 2); maxIt = 1000; tol = 1e-6
output = DEoptim(fn = MRCobjN, lower = lb, upper = ub, 
                 control = list(itermax = maxIt/5, trace = T, reltol = tol, steptol = maxIt/10))
c(output$optim$bestmem, output$optim$bestval) 
  
# optimx with many random initial values
# set.seed(r)
# b0 = runif(2, 0, 2); tol = 1e-6
# output = optimx(b0, fn = MRCobjN, method = c("Nelder-Mead"), control = list(trace = T, reltol = tol))
# c(output$p1, output$p2, output$value)

}

# DEoptim searching for global optimum 
MRC.estimation(1)

# optimx with 100 random initial values
# R = 100
# cs.res = matrix(unlist(sapply(1:R, MRC.estimation), use.names = F), ncol = 2+1, byrow = T)
# cs.res = cs.res[order(cs.res[,3], decreasing = F),]
# cs.res[1,]

# DEoptim searching for global optimum 
# (0.01661, 0.119181)


