
rm(list = ls())

library(parallel)
library(DEoptim)
library(mvtnorm)
library(optimx)

# paper using Cracker data
# Paap and Frances (2000), Journal of Applied Econometrics.
# Jain, Vilcassim and Chintagunta (1994), JBES.
library(mlogit)
data("Cracker", package = "mlogit")
library(dplyr)
Cracker = filter(Cracker, price.nabisco!=0 & price.sunshine!=0 & 
                 price.keebler!=0 & price.private!=0)
attach(Cracker)

# sample size and number of periods
N = length(unique(id))
TT = table(id); max(TT); min(TT)

# dependent variable and regressors
y = seq(0, 0, length = nrow(Cracker))

normalize = function(x) {
  log(x/100)
}

y[choice == "nabisco"] = 1; x11 = normalize(price.nabisco); x12 = disp.nabisco; x13 = feat.nabisco
y[choice == "sunshine"] = 2; x21 = normalize(price.sunshine); x22 = disp.sunshine; x23 = feat.sunshine
y[choice == "keebler"] = 3; x31 = normalize(price.keebler); x32 = disp.keebler; x33 = feat.keebler
y[choice == "private"] = 4; x41 = normalize(price.private); x42 = disp.private; x43 = feat.private

dataset = cbind(id, y, x11, x12, x13, x21, x22, x23, x31, x32, x33, x41, x42, x43)

# bandwidth 
c1 = 2*sd(x11); c2 = 2*sd(x21); c3 = 2*sd(x31); c4 = 2*sd(x41)
hn1 = 5*c1*N^(-3/29); hn2 = 5*c2*N^(-3/29); hn3 = 5*c3*N^(-3/29); hn4 = 5*c4*N^(-3/29)
gn1 = 5*c1*log(N)^(-1/3)*N^(-1/6); gn2 = 5*c2*log(N)^(-1/3)*N^(-1/6)
gn3 = 5*c3*log(N)^(-1/3)*N^(-1/6); gn4 = 5*c4*log(N)^(-1/3)*N^(-1/6) 

# MS estimation
MS.estimation = function(dataset) {
  
  # MS objective function 
  MSobjN = function(b) {
    
    b1 = b[1]; b2 = b[2]; r = b[3]
    
    score1.i = function(i) {
      
      data.i = dataset[id == i,][,2:ncol(dataset)]
      
      yi = data.i[,1]; Ti = length(yi)
      x11i = data.i[,2]; x12i = data.i[,3]; x13i = data.i[,4]
      x21i = data.i[,5]; x22i = data.i[,6]; x23i = data.i[,7]
      x31i = data.i[,8]; x32i = data.i[,9]; x33i = data.i[,10]
      x41i = data.i[,11]; x42i = data.i[,12]; x43i = data.i[,13]
      
      # case: s = t + 1
      Sa.it = function(t) {
        
        yi0 = yi[t-1]; yi1 = yi[t]; yi2 = yi[t+1]; yi3 = yi[t+2]
        
        x11i1 = x11i[t]; x12i1 = x12i[t]; x13i1 = x13i[t]
        x21i1 = x21i[t]; x22i1 = x22i[t]; x23i1 = x23i[t]
        x31i1 = x31i[t]; x32i1 = x32i[t]; x33i1 = x33i[t]
        x41i1 = x41i[t]; x42i1 = x42i[t]; x43i1 = x43i[t]
        
        x11i2 = x11i[t+1]; x12i2 = x12i[t+1]; x13i2 = x13i[t+1]
        x21i2 = x21i[t+1]; x22i2 = x22i[t+1]; x23i2 = x23i[t+1]
        x31i2 = x31i[t+1]; x32i2 = x32i[t+1]; x33i2 = x33i[t+1]
        x41i2 = x41i[t+1]; x42i2 = x42i[t+1]; x43i2 = x43i[t+1]
        
        x11i3 = x11i[t+2]; x12i3 = x12i[t+2]; x13i3 = x13i[t+2]
        x21i3 = x21i[t+2]; x22i3 = x22i[t+2]; x23i3 = x23i[t+2]
        x31i3 = x31i[t+2]; x32i3 = x32i[t+2]; x33i3 = x33i[t+2]
        x41i3 = x41i[t+2]; x42i3 = x42i[t+2]; x43i3 = x43i[t+2]
        
        # match all x in t+1 and t+2
        K1i.23 = dnorm((x11i2-x11i3)/hn1)*dnorm((x21i2-x21i3)/hn2)*
                 dnorm((x31i2-x31i3)/hn3)*dnorm((x41i2-x41i3)/hn4)*
                 (x12i2 == x12i3)*(x22i2 == x22i3)*(x32i2 == x32i3)*(x42i2 == x42i3)*
                 (x13i2 == x13i3)*(x23i2 == x23i3)*(x33i2 == x33i3)*(x43i2 == x43i3)
        
        # choice 1, match {2,3,4} in t and t+1
        K1i.12 = dnorm((x21i1-x21i2)/hn2)*(x22i1 == x22i2)*(x23i1 == x23i2)*
                 dnorm((x31i1-x31i2)/hn3)*(x32i1 == x32i2)*(x33i1 == x33i2)*
                 dnorm((x41i1-x41i2)/hn4)*(x42i1 == x42i2)*(x43i1 == x43i2)
        
        d1i0 = (yi0 == 1); d1i3 = (yi3 == 1)
        U1i.12 = sign(-x11i1 + b1*x12i1 + b2*x13i1 + r*d1i0 - (-x11i2 + b1*x12i2 + b2*x13i2 + r*d1i3))
        AB = (yi1 == 1)-(yi2 == 1)
        
        (-1)*K1i.12*K1i.23*AB*U1i.12
        
      }
      
      # case: s > t + 1
      Sna.it = function(t) {
        
        Sna.its = function(s) {
          
          x11i1 = x11i[t]; x12i1 = x12i[t]; x13i1 = x13i[t]
          x21i1 = x21i[t]; x22i1 = x22i[t]; x23i1 = x23i[t]
          x31i1 = x31i[t]; x32i1 = x32i[t]; x33i1 = x33i[t]
          x41i1 = x41i[t]; x42i1 = x42i[t]; x43i1 = x43i[t]
          
          x11i2 = x11i[t+1]; x12i2 = x12i[t+1]; x13i2 = x13i[t+1]
          x21i2 = x21i[t+1]; x22i2 = x22i[t+1]; x23i2 = x23i[t+1]
          x31i2 = x31i[t+1]; x32i2 = x32i[t+1]; x33i2 = x33i[t+1]
          x41i2 = x41i[t+1]; x42i2 = x42i[t+1]; x43i2 = x43i[t+1]
          
          x11i3 = x11i[s]; x12i3 = x12i[s]; x13i3 = x13i[s]
          x21i3 = x21i[s]; x22i3 = x22i[s]; x23i3 = x23i[s]
          x31i3 = x31i[s]; x32i3 = x32i[s]; x33i3 = x33i[s]
          x41i3 = x41i[s]; x42i3 = x42i[s]; x43i3 = x43i[s]
          
          x11i4 = x11i[s+1]; x12i4 = x12i[s+1]; x13i4 = x13i[s+1]
          x21i4 = x21i[s+1]; x22i4 = x22i[s+1]; x23i4 = x23i[s+1]
          x31i4 = x31i[s+1]; x32i4 = x32i[s+1]; x33i4 = x33i[s+1]
          x41i4 = x41i[s+1]; x42i4 = x42i[s+1]; x43i4 = x43i[s+1]
          
          # match all x in t+1 and s+1
          K1i.23 = dnorm((x11i2-x11i4)/hn1)*dnorm((x21i2-x21i4)/hn2)*
                   dnorm((x31i2-x31i4)/hn3)*dnorm((x41i2-x41i4)/hn4)*
                   (x12i2 == x12i4)*(x22i2 == x22i4)*(x32i2 == x32i4)*(x42i2 == x42i4)*
                   (x13i2 == x13i4)*(x23i2 == x23i4)*(x33i2 == x33i4)*(x43i2 == x43i4)
          
          # choice 1, match {2,3,4} in t and s
          K1i.12 = dnorm((x21i1-x21i3)/hn2)*(x22i1 == x22i3)*(x23i1 == x23i3)*
                   dnorm((x31i1-x31i3)/hn3)*(x32i1 == x32i3)*(x33i1 == x33i3)*
                   dnorm((x41i1-x41i3)/hn4)*(x42i1 == x42i3)*(x43i1 == x43i3)
          
          U1i.12 = sign((-x11i1 + b1*x12i1 + b2*x13i1 + r*(yi[t-1]==1)) - 
                        (-x11i3 + b1*x12i3 + b2*x13i3 + r*(yi[s-1]==1)))
          AB = (yi[t]==1)-(yi[s]==1)
        
          (-1)*K1i.12*K1i.23*AB*U1i.12*(yi[s+1]==yi[t+1])
        
        }
        
        sum(sapply((t+2):(Ti-1), Sna.its))
        
      }
          
      sum(sapply(2:(Ti-2), Sa.it)) + sum(sapply(2:(Ti-3), Sna.it)) 
      
    }
   
    Score1 = sum(sapply(1:N, score1.i))
    
    score234.i = function(i) {
      
      data.i = dataset[id == i,][,2:ncol(dataset)]
      
      yi = data.i[,1]; yi_1 = c(NA, yi[1:(length(yi)-1)])
      x11i = data.i[,2]; x12i = data.i[,3]; x13i = data.i[,4]
      x21i = data.i[,5]; x22i = data.i[,6]; x23i = data.i[,7]
      x31i = data.i[,8]; x32i = data.i[,9]; x33i = data.i[,10]
      x41i = data.i[,11]; x42i = data.i[,12]; x43i = data.i[,13]
      
      Ti = length(yi); CTi = combn(Ti, 2); CombTi = split(CTi, col(CTi))
      
      # create paired data
      CombData.i = function(ts) {
        t = ts[1]; s = ts[2]
        c(yi[t], x11i[t], x12i[t], x13i[t], x21i[t],x22i[t],x23i[t],
          x31i[t],x32i[t],x33i[t],x41i[t],x42i[t],x43i[t],yi_1[t],
          yi[s], x11i[s], x12i[s], x13i[s], x21i[s],x22i[s],x23i[s],
          x31i[s],x32i[s],x33i[s],x41i[s],x42i[s],x43i[s],yi_1[s])
      }
      
      pairedData.i = t(sapply(CombTi, CombData.i))
      pairedData.i = pairedData.i[(is.na(pairedData.i[,14]) == F),]
      
      # parts in objective function not depending on preference parameters
      data.it = pairedData.i[,1:14]; data.is = pairedData.i[,15:28]
      
      yit = data.it[,1]; yit_1 = data.it[,14]
      x11it = data.it[,2]; x12it = data.it[,3]; x13it = data.it[,4]
      x21it = data.it[,5]; x22it = data.it[,6]; x23it = data.it[,7]
      x31it = data.it[,8]; x32it = data.it[,9]; x33it = data.it[,10]
      x41it = data.it[,11]; x42it = data.it[,12]; x43it = data.it[,13]
      
      yis = data.is[,1]; yis_1 = data.is[,14]
      x11is = data.is[,2]; x12is = data.is[,3]; x13is = data.is[,4]
      x21is = data.is[,5]; x22is = data.is[,6]; x23is = data.is[,7]
      x31is = data.is[,8]; x32is = data.is[,9]; x33is = data.is[,10]
      x41is = data.is[,11]; x42is = data.is[,12]; x43is = data.is[,13]
      
      # choice 2, match choice {1,3,4}  
      K2its = dnorm((x11it-x11is)/gn1)*(x12it == x12is)*(x13it == x13is)*((yit_1==1)==(yis_1==1))*
              dnorm((x31it-x31is)/gn3)*(x32it == x32is)*(x33it == x33is)*
              dnorm((x41it-x41is)/gn4)*(x42it == x42is)*(x43it == x43is)
      
      R2its = ((yis==1)-(yit==1)) + ((yit==2)-(yis==2)) + 
              ((yis==3)-(yit==3)) + ((yis==4)-(yit==4)) 
      
      # choice 3, match choice {1,2,4}
      K3its = dnorm((x11it-x11is)/gn1)*(x12it == x12is)*(x13it == x13is)*((yit_1==1)==(yis_1==1))*
              dnorm((x21it-x21is)/gn2)*(x22it == x22is)*(x23it == x23is)*
              dnorm((x41it-x41is)/gn4)*(x42it == x42is)*(x43it == x43is)
      
      R3its = ((yis==1)-(yit==1)) + ((yis==2)-(yit==2)) + 
              ((yit==3)-(yis==3)) + ((yit==4)-(yis==4)) 
      
      # choice 4, match choice {1,2,3}
      K4its = dnorm((x11it-x11is)/gn1)*(x12it == x12is)*(x13it == x13is)*((yit_1==1)==(yis_1==1))*
              dnorm((x21it-x21is)/gn2)*(x22it == x22is)*(x23it == x23is)*
              dnorm((x31it-x31is)/gn3)*(x32it == x32is)*(x33it == x33is)
      
      R4its = ((yis==1)-(yit==1)) + ((yis==2)-(yit==2)) + 
              ((yis==3)-(yit==3)) + ((yit==4)-(yis==4)) 
      
      U2it = -x21it + x22it*b1 + x23it*b2; U3it = -x31it + x32it*b1 + x33it*b2
      U4it = -x41it + x42it*b1 + x43it*b2
      
      U2is = -x21is + x22is*b1 + x23is*b2; U3is = -x31is + x32is*b1 + x33is*b2
      U4is = -x41is + x42is*b1 + x43is*b2
      
      (-1)*sum(K2its*R2its*sign(U2it-U2is) + K3its*R3its*sign(U3it-U3is) + K4its*R4its*sign(U4it-U4is))
      
    }
    
    Score234 = sum(sapply(1:N, score234.i))
    
    Score1 + Score234

  }
  
  ## DEoptim 
  lb = seq(0, 0, length = 3); ub = seq(2, 2, length = 3); maxIt = 500; tol = 1e-6
  output = DEoptim(fn = MSobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/2, trace = T, reltol = tol, steptol = maxIt/10))
  c(output$optim$bestmem, output$optim$bestval)
  
}

MS.estimation(dataset)

# estimates = (0.1357523, 0.2460285, 0.2511690), criterion function = -62.3609573 


