
rm(list = ls())

library(parallel)
library(optimx)
library(dplyr)

# paper using Cracker data
# Paap and Frances (2000), Journal of Applied Econometrics.
# Jain, Vilcassim and Chintagunta (1994), JBES.
library(mlogit)
data("Cracker", package = "mlogit")
Cracker = filter(Cracker, price.nabisco!=0 & price.sunshine!=0 & 
                   price.keebler!=0 & price.private!=0)
attach(Cracker)

# sample size and number of periods
N = length(unique(id))
TT = table(id); max(TT); min(TT)

# dependent variable and regressors
y = seq(0, 0, length = nrow(Cracker))

normalize = function(x) {
  log(x/100)
}

y[choice == "nabisco"] = 1; x11 = normalize(price.nabisco); x12 = disp.nabisco; x13 = feat.nabisco
y[choice == "sunshine"] = 2; x21 = normalize(price.sunshine); x22 = disp.sunshine; x23 = feat.sunshine
y[choice == "keebler"] = 3; x31 = normalize(price.keebler); x32 = disp.keebler; x33 = feat.keebler
y[choice == "private"] = 4; x41 = normalize(price.private); x42 = disp.private; x43 = feat.private

# bandwidth 
# hn = N^(-3/29)
c1 = 2*sd(x11); c2 = 2*sd(x21); c3 = 2*sd(x31); c4 = 2*sd(x41)
hn1 = 5*c1*N^(-3/29); hn2 = 5*c2*N^(-3/29); hn3 = 5*c3*N^(-3/29); hn4 = 5*c4*N^(-3/29)

dataset = cbind(id, y, x11, x12, x13, x21, x22, x23, x31, x32, x33, x41, x42, x43)

# ML estimation
ML.estimation = function(dataset) {
  
  # ML objective function 
  MLobjN = function(b) {
    
    b1 = b[1]; b2 = b[2]; b3 = b[3]; r = b[4]
    
    score1.i = function(i) {
      
      data.i = dataset[id == i,][,2:ncol(dataset)]
      
      yi = data.i[,1]; Ti = length(yi)
      x11i = data.i[,2]; x12i = data.i[,3]; x13i = data.i[,4]
      x21i = data.i[,5]; x22i = data.i[,6]; x23i = data.i[,7]
      x31i = data.i[,8]; x32i = data.i[,9]; x33i = data.i[,10]
      x41i = data.i[,11]; x42i = data.i[,12]; x43i = data.i[,13]
      
      S.it = function(t) {
        
        S.its = function(s) {
          
          x11i1 = x11i[t]; x12i1 = x12i[t]; x13i1 = x13i[t]
          x21i1 = x21i[t]; x22i1 = x22i[t]; x23i1 = x23i[t]
          x31i1 = x31i[t]; x32i1 = x32i[t]; x33i1 = x33i[t]
          x41i1 = x41i[t]; x42i1 = x42i[t]; x43i1 = x43i[t]
          
          x11i2 = x11i[t+1]; x12i2 = x12i[t+1]; x13i2 = x13i[t+1]
          x21i2 = x21i[t+1]; x22i2 = x22i[t+1]; x23i2 = x23i[t+1]
          x31i2 = x31i[t+1]; x32i2 = x32i[t+1]; x33i2 = x33i[t+1]
          x41i2 = x41i[t+1]; x42i2 = x42i[t+1]; x43i2 = x43i[t+1]
          
          x11i3 = x11i[s]; x12i3 = x12i[s]; x13i3 = x13i[s]
          x21i3 = x21i[s]; x22i3 = x22i[s]; x23i3 = x23i[s]
          x31i3 = x31i[s]; x32i3 = x32i[s]; x33i3 = x33i[s]
          x41i3 = x41i[s]; x42i3 = x42i[s]; x43i3 = x43i[s]
          
          x11i4 = x11i[s+1]; x12i4 = x12i[s+1]; x13i4 = x13i[s+1]
          x21i4 = x21i[s+1]; x22i4 = x22i[s+1]; x23i4 = x23i[s+1]
          x31i4 = x31i[s+1]; x32i4 = x32i[s+1]; x33i4 = x33i[s+1]
          x41i4 = x41i[s+1]; x42i4 = x42i[s+1]; x43i4 = x43i[s+1]
          
          # match all x in t+1 and s+1
          K1i.23 = dnorm((x11i2-x11i4)/hn1)*dnorm((x21i2-x21i4)/hn2)*
                   dnorm((x31i2-x31i4)/hn3)*dnorm((x41i2-x41i4)/hn4)*
                   (x12i2 == x12i4)*(x22i2 == x22i4)*(x32i2 == x32i4)*(x42i2 == x42i4)*
                   (x13i2 == x13i4)*(x23i2 == x23i4)*(x33i2 == x33i4)*(x43i2 == x43i4)
          
          # choice 1, match {2,3,4} in t and s
          K1i.12 = dnorm((x21i1-x21i3)/hn2)*(x22i1 == x22i3)*(x23i1 == x23i3)*
                   dnorm((x31i1-x31i3)/hn3)*(x32i1 == x32i3)*(x33i1 == x33i3)*
                   dnorm((x41i1-x41i3)/hn4)*(x42i1 == x42i3)*(x43i1 == x43i3)
          
          U1i.12 = (x11i1*b1 + x12i1*b2 + x13i1*b3) - (x11i3*b1 + x12i3*b2 + x13i3*b3) +
            r*((yi[t-1]==1) - (yi[s+1]==1)) + r*((yi[t+1]==1) - (yi[s-1]==1))*(s-t > 1)
          AB = ((yi[t]==1)+(yi[s]==1)==1)
          
          (-1)*K1i.12*K1i.23*AB*log(1e-8 + exp(U1i.12)^(yi[t]==1)/(1 + exp(U1i.12)))*(s-t > 0)
          
        }
        
        sum(sapply(2:(Ti-1), S.its))
        
      }
      
      sum(sapply(2:(Ti-1), S.it)) 
      
    }
    
    sum(sapply(1:N, score1.i))
  # sum(sapply(1:N, score1.i)) + sum(sapply(1:N, score234.i))
    
  }
  
  b0 = runif(4, -2, 2); maxIt = 500; tol = 1e-8
  output = optimx(b0, MLobjN, method = c("Nelder-Mead"), itnmax = maxIt, control = list(trace = T))
  c(output$p1, output$p2, output$p3, output$p4, -output$p2/output$p1, -output$p3/output$p1, -output$p4/output$p1)
  
}

ML.estimation(dataset)

# estimates = (-0.1565825, 0.2745558, 0.2288419)



