
rm(list = ls())

library(parallel)
library(DEoptim)
library(mvtnorm)
library(optimx)

# paper using Cracker data
# Paap and Frances (2000), Journal of Applied Econometrics.
# Jain, Vilcassim and Chintagunta (1994), JBES.
library(mlogit)
data("Cracker", package = "mlogit")
library(dplyr)
Cracker = filter(Cracker, price.nabisco!=0 & price.sunshine!=0 & 
                   price.keebler!=0 & price.private!=0)
attach(Cracker)

# sample size and number of periods
N = length(unique(id))
TT = table(id); max(TT); min(TT)

# dependent variable and regressors
y = seq(0, 0, length = nrow(Cracker))

normalize = function(x) {
  # price = c(price.nabisco, price.sunshine, price.keebler, price.private)
  # (x - min(price))/(max(price) - min(price))
  # (x - mean(price))/sd(price)
  # log((x+100)/100)
  # (x - mean(price))/100
  # x/100
  log(x/100)
}

y[choice == "nabisco"]  = 1; x11 = normalize(price.nabisco); x12 = disp.nabisco; x13 = feat.nabisco
y[choice == "sunshine"] = 2; x21 = normalize(price.sunshine); x22 = disp.sunshine; x23 = feat.sunshine
y[choice == "keebler"]  = 3; x31 = normalize(price.keebler); x32 = disp.keebler; x33 = feat.keebler
y[choice == "private"]  = 4; x41 = normalize(price.private); x42 = disp.private; x43 = feat.private

dataset = cbind(id, y, x11, x12, x13, x21, x22, x23, x31, x32, x33, x41, x42, x43)
head(dataset)

# bandwidth 
hn = 3*log(N)^(-1/3)*N^(-1/6) 
c1 = sd(x11); c2 = sd(x21); c3 = sd(x31); c4 = sd(x41)
hn1 = c1*hn; hn2 = c2*hn; hn3 = c3*hn; hn4 = c4*hn

# MS estimation
MS.estimation = function(r) {
  
  # MS objective function 
  MSobjN = function(b) {
    
    score.i = function(i) {
      
      data.i = dataset[id == i,][,2:ncol(dataset)]
    
      yi = data.i[,1]
      x11i = data.i[,2]; x12i = data.i[,3]; x13i = data.i[,4]; x21i = data.i[,5]
      x22i = data.i[,6]; x23i = data.i[,7]; x31i = data.i[,8]; x32i = data.i[,9]
      x33i = data.i[,10]; x41i = data.i[,11]; x42i = data.i[,12]; x43i = data.i[,13]
    
      Ti = length(yi); CTi = combn(Ti, 2); CombTi = split(CTi, col(CTi))
    
      # create paired data
      CombData.i = function(ts) {
        t = ts[1]; s = ts[2]
        c(yi[t],x11i[t],x12i[t],x13i[t],x21i[t],x22i[t],x23i[t],x31i[t],x32i[t],x33i[t],x41i[t],x42i[t],x43i[t],
          yi[s],x11i[s],x12i[s],x13i[s],x21i[s],x22i[s],x23i[s],x31i[s],x32i[s],x33i[s],x41i[s],x42i[s],x43i[s])
      }
    
      pairedData.i = t(sapply(CombTi, CombData.i))
    
      # parts in objective function not depending on preference parameters
      data.it = pairedData.i[,1:13]; data.is = pairedData.i[,14:26]
    
      yit = data.it[,1]
      x11it = data.it[,2]; x12it = data.it[,3]; x13it = data.it[,4]
      x21it = data.it[,5]; x22it = data.it[,6]; x23it = data.it[,7]
      x31it = data.it[,8]; x32it = data.it[,9]; x33it = data.it[,10]
      x41it = data.it[,11]; x42it = data.it[,12]; x43it = data.it[,13]
    
      yis = data.is[,1]
      x11is = data.is[,2]; x12is = data.is[,3]; x13is = data.is[,4]
      x21is = data.is[,5]; x22is = data.is[,6]; x23is = data.is[,7]
      x31is = data.is[,8]; x32is = data.is[,9]; x33is = data.is[,10]
      x41is = data.is[,11]; x42is = data.is[,12]; x43is = data.is[,13]
      
      yi1t = (yit==1); yi2t = (yit==2); yi3t = (yit==3); yi4t = (yit==4)   
      yi1s = (yis==1); yi2s = (yis==2); yi3s = (yis==3); yi4s = (yis==4)  
    
      # choice 1, match choice {2,3,4}
      K1its = dmvnorm(cbind((x41it-x41is)/hn4,(x21it-x21is)/hn2,(x31it-x31is)/hn3))*
        (x22it == x22is)*(x23it == x23is)*
        (x32it == x32is)*(x33it == x33is)*(x42it == x42is)*(x43it == x43is)
      R1its = (yi1t-yi1s) + (yi2s-yi2t) + (yi3s-yi3t) + (yi4s-yi4t)
    
      # choice 2, match choice {1,3,4}  
      K2its = dmvnorm(cbind((x11it-x11is)/hn1,(x41it-x41is)/hn4,(x31it-x31is)/hn3))*
        (x12it == x12is)*(x13it == x13is)*
        (x32it == x32is)*(x33it == x33is)*(x42it == x42is)*(x43it == x43is)
      R2its = (yi1s-yi1t) + (yi2t-yi2s) + (yi3s-yi3t) + (yi4s-yi4t)
    
      # choice 3, match choice {1,2,4}
      K3its = dmvnorm(cbind((x11it-x11is)/hn1,(x21it-x21is)/hn2,(x41it-x41is)/hn4))*
        (x12it == x12is)*(x13it == x13is)*
        (x22it == x22is)*(x23it == x23is)*(x42it == x42is)*(x43it == x43is)
      R3its = (yi1s-yi1t) + (yi2s-yi2t) + (yi3t-yi3s) + (yi4s-yi4t)
    
      # choice 4, match choice {1,2,3}
      K4its = dmvnorm(cbind((x11it-x11is)/hn1,(x21it-x21is)/hn2,(x31it-x31is)/hn3))*
        (x12it == x12is)*(x13it == x13is)*
        (x22it == x22is)*(x23it == x23is)*(x32it == x32is)*(x33it == x33is)
      R4its = (yi1s-yi1t) + (yi2s-yi2t) + (yi3s-yi3t) + (yi4t-yi4s)
    
      U1it = (-1)*x11it + x12it*b[1] + x13it*b[2]; U2it = (-1)*x21it + x22it*b[1] + x23it*b[2]
      U3it = (-1)*x31it + x32it*b[1] + x33it*b[2]; U4it = (-1)*x41it + x42it*b[1] + x43it*b[2]
      U1is = (-1)*x11is + x12is*b[1] + x13is*b[2]; U2is = (-1)*x21is + x22is*b[1] + x23is*b[2]
      U3is = (-1)*x31is + x32is*b[1] + x33is*b[2]; U4is = (-1)*x41is + x42is*b[1] + x43is*b[2]
      
      Score.i = K1its*R1its*sign(U1it - U1is) + K2its*R2its*sign(U2it - U2is) +
                K3its*R3its*sign(U3it - U3is) + K4its*R4its*sign(U4it - U4is)
      
      (-1)*sum(Score.i)
    
    }
    
    sum(sapply(1:N, score.i))/N
  
  }
  
  # DEoptim searching for global optimum 
  lb = seq(-1, -1, length = 2); ub = seq(2, 2, length = 2); maxIt = 400; tol = 1e-6
  output = DEoptim(fn = MSobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/2, trace = T, reltol = tol, steptol = maxIt/5))
  c(output$optim$bestmem, output$optim$bestval)
  
  # optimx with many random initial values
  # set.seed(r)
  # b0 = runif(2, 0, 2)
  # output = optimx(b0, MSobjN, method = c("Nelder-Mead"), control = list(trace = T))
  # c(output$p1, output$p2, output$value)
  
}

# DEoptim searching global optimum 
MS.estimation(1)
# results: (0.06390162, 0.08220999), obj = -0.99134193 

# optimx with 100 random initial values
# R = 100
# sp.res = matrix(unlist(sapply(1:R, MS.estimation), use.names = F), ncol = 2+1, byrow = T)
# sp.res = sp.res[order(sp.res[,3], decreasing = F),]
# sp.res[1,]

