
rm(list = ls())

library(parallel)
library(optimx)
library(dplyr)

# paper using Cracker data
# Paap and Frances (2000), Journal of Applied Econometrics.
# Jain, Vilcassim and Chintagunta (1994), JBES.
library(mlogit)
data("Cracker", package = "mlogit")
Cracker = filter(Cracker, price.nabisco!=0 & price.sunshine!=0 & 
                   price.keebler!=0 & price.private!=0)
attach(Cracker)

# sample size and number of periods
N = length(unique(id))
TT = table(id); max(TT); min(TT)

# dependent variable and regressors
y = seq(0, 0, length = nrow(Cracker))

normalize = function(x) {
  log(x/100)
}

y[choice == "nabisco"] = 1; x11 = normalize(price.nabisco); x12 = disp.nabisco; x13 = feat.nabisco
y[choice == "sunshine"] = 2; x21 = normalize(price.sunshine); x22 = disp.sunshine; x23 = feat.sunshine
y[choice == "keebler"] = 3; x31 = normalize(price.keebler); x32 = disp.keebler; x33 = feat.keebler
y[choice == "private"] = 4; x41 = normalize(price.private); x42 = disp.private; x43 = feat.private

dataset = cbind(id, y, x11, x12, x13, x21, x22, x23, x31, x32, x33, x41, x42, x43)

# ML estimation
ML.estimation = function(dataset) {
  
  # ML objective function 
  MLobjN = function(b) {
    
    score.i = function(i) {
      
      data.i = dataset[id == i,][,2:ncol(dataset)]
      
      yi =   data.i[,1]
      x11i = data.i[,2];  x12i = data.i[,3];  x13i = data.i[,4]
      x21i = data.i[,5];  x22i = data.i[,6];  x23i = data.i[,7]
      x31i = data.i[,8];  x32i = data.i[,9];  x33i = data.i[,10]
      x41i = data.i[,11]; x42i = data.i[,12]; x43i = data.i[,13]
      
      Ti = length(yi); CTi = combn(Ti, 2); CombTi = split(CTi, col(CTi))
      
      # create paired data
      CombData.i = function(ts) {
        t = ts[1]; s = ts[2]
        c(yi[t], x11i[t], x12i[t], x13i[t], x21i[t], x22i[t], x23i[t],
          x31i[t], x32i[t], x33i[t], x41i[t], x42i[t], x43i[t],
          yi[s], x11i[s], x12i[s], x13i[s], x21i[s], x22i[s], x23i[s],
          x31i[s], x32i[s], x33i[s], x41i[s], x42i[s], x43i[s])
      }
      
      pairedData.i = t(sapply(CombTi, CombData.i))
      
      # parts in objective function not depending on preference parameters
      data.it = pairedData.i[,1:13]; data.is = pairedData.i[,14:26]
      
      yit = data.it[,1]; x11it = data.it[,2]; x12it = data.it[,3]; x13it = data.it[,4]
      x21it = data.it[,5]; x22it = data.it[,6]; x23it = data.it[,7]
      x31it = data.it[,8]; x32it = data.it[,9]; x33it = data.it[,10]
      x41it = data.it[,11]; x42it = data.it[,12]; x43it = data.it[,13]
      
      yis = data.is[,1]; x11is = data.is[,2]; x12is = data.is[,3]; x13is = data.is[,4]
      x21is = data.is[,5]; x22is = data.is[,6]; x23is = data.is[,7]
      x31is = data.is[,8]; x32is = data.is[,9]; x33is = data.is[,10]
      x41is = data.is[,11]; x42is = data.is[,12]; x43is = data.is[,13]
      
      U1it = x11it*b[1] + x12it*b[2] + x13it*b[3]; U2it = x21it*b[1] + x22it*b[2] + x23it*b[3]
      U3it = x31it*b[1] + x32it*b[2] + x33it*b[3]; U4it = x41it*b[1] + x42it*b[2] + x43it*b[3]
      
      U1is = x11is*b[1] + x12is*b[2] + x13is*b[3]; U2is = x21is*b[1] + x22is*b[2] + x23is*b[3]
      U3is = x31is*b[1] + x32is*b[2] + x33is*b[3]; U4is = x41is*b[1] + x42is*b[2] + x43is*b[3]
      
      D12 = (yit == 1)*(yis == 2); D21 = (yit == 2)*(yis == 1)
      D13 = (yit == 1)*(yis == 3); D31 = (yit == 3)*(yis == 1)
      D14 = (yit == 1)*(yis == 4); D41 = (yit == 4)*(yis == 1)
      D23 = (yit == 2)*(yis == 3); D32 = (yit == 3)*(yis == 2)
      D24 = (yit == 2)*(yis == 4); D42 = (yit == 4)*(yis == 2)
      D34 = (yit == 3)*(yis == 4); D43 = (yit == 4)*(yis == 3)
      
      D1 = U1it - U1is; D2 = U2it - U2is; D3 = U3it - U3is; D4 = U4it - U4is 
      
      Score.i = (D12 + D21)*log(1e-8 + exp(D2 - D1)^D21/(1 + exp(D2 - D1))) +
                (D13 + D31)*log(1e-8 + exp(D3 - D1)^D31/(1 + exp(D3 - D1))) +
                (D14 + D41)*log(1e-8 + exp(D4 - D1)^D41/(1 + exp(D4 - D1))) +
                (D23 + D32)*log(1e-8 + exp(D3 - D2)^D32/(1 + exp(D3 - D2))) +
                (D24 + D42)*log(1e-8 + exp(D4 - D2)^D42/(1 + exp(D4 - D2))) +
                (D34 + D43)*log(1e-8 + exp(D4 - D3)^D43/(1 + exp(D4 - D3))) 
      
      (-1)*sum(Score.i)
      
    }
    
    sum(sapply(1:N, score.i))
    
  }
  
  b0 = runif(3, -2, 2); maxIt = 500; tol = 1e-8
  output = optimx(b0, MLobjN, method = "BFGS", itnmax = maxIt, hessian = T, control = list(trace = T))
  c(output$p1, output$p2, output$p3, -output$p2/output$p1, -output$p3/output$p1)
  
}

ML.estimation(dataset)


