
##################
# MRC Simulation #
##################

rm(list = ls())

library(MASS)
library(parallel)
library(DEoptim)
library(mvtnorm)

path = "/Users/uqfouyan/Dropbox/Papers/my papers/KOT_2019_multinomial/simulation/continuous"
setwd(path) 

# detect the number of physical CPU cores for parallel computation
n.cores = detectCores(logical = F) 

# DGP: choice set = {0,1,2}
# Uj = xj1 + xj2*b1 + xj3*b2 - ej, j = 1,2
# U0 = 0 (outside/base choice)
J = 3                             # number of choices
K = 3                             # number of regressors
coefdgp = c(1, 1)                 # true parameters

# sample size
N = 1000
CN = combn(N, 2); CombN = split(CN, col(CN))

# bandwidth 
a = 1.0; hn = a*N^(-1/6)

# MRC simulation
MRC.simu = function(s) {
  
  # generate random sample from DGP
  set.seed(s)
  mu = seq(0, 0, length = J-1)
  sigma = matrix(0.5*seq(1, 1, length = (J-1)^2), nrow = J-1) + diag(J-1)*0.5
  e = mvrnorm(n = N, mu, sigma) 
  x11 = rnorm(N, 0, 1); x12 = runif(N, -1, 1); x13 = runif(N, -1, 1)
  x21 = rnorm(N, 0, 1); x22 = runif(N, -1, 1); x23 = runif(N, -1, 1)
  b0 = coefdgp
  u0 = 0
  u1 = x11 + b0[1]*x12 + b0[2]*x13 - e[,1]
  u2 = x21 + b0[1]*x22 + b0[2]*x23 - e[,2]
  y = 0*(u1 < u0)*(u2 < u0) + 1*(u0 < u1)*(u2 < u1) + 2*(u1 < u2)*(u0 < u2)
  
  # create paired data
  CombData = function(im) {
    i = im[1]; m = im[2]
    c(y[i], x11[i], x12[i], x13[i], x21[i], x22[i], x23[i],
      y[m], x11[m], x12[m], x13[m], x21[m], x22[m], x23[m])
  }
  
  paired.Data = t(sapply(CombN, CombData))
  
  data_i = paired.Data[,1:(ncol(paired.Data)/2)] 
  data_m = paired.Data[,(1+ncol(paired.Data)/2):(ncol(paired.Data))]
  
  yi = data_i[,1]
  x11i = data_i[,2];  x12i = data_i[,3];  x13i = data_i[,4]
  x21i = data_i[,5];  x22i = data_i[,6];  x23i = data_i[,7]
  
  ym = data_m[,1]
  x11m = data_m[,2];  x12m = data_m[,3];  x13m = data_m[,4]
  x21m = data_m[,5];  x22m = data_m[,6];  x23m = data_m[,7]
  
  # choice 1, match choice 2
  v1im = cbind(x21i-x21m, x22i-x22m, x23i-x23m)
  K1im = dmvnorm(v1im/hn)
  R1im = ((ym==0) - (yi==0)) + ((yi==1) - (ym==1)) + ((ym==2) - (yi==2))
  
  # choice 2, match choice 1 
  v2im = cbind(x11i-x11m, x12i-x12m, x13i-x13m)
  K2im = dmvnorm(v2im/hn)
  R2im = ((ym==0) - (yi==0)) + ((ym==1) - (yi==1)) + ((yi==2) - (ym==2)) 
  
  # MRC objective function 
  MRCobjN = function(b) {
    
    U1i = x11i + x12i*b[1] + x13i*b[2]; U2i = x21i + x22i*b[1] + x23i*b[2]
    U1m = x11m + x12m*b[1] + x13m*b[2]; U2m = x21m + x22m*b[1] + x23m*b[2]
    
    Score = K1im*R1im*sign(U1i - U1m) + K2im*R2im*sign(U2i - U2m) 
    2*(-1)*mean(Score)
  }
  
  # DEoptim
  lb = seq(0, 0, length = 2); ub = seq(2, 2, length = 2); maxIt = 1000; tol = 1e-6
  output = DEoptim(fn = MRCobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/5, trace = T, reltol = tol, steptol = maxIt/20))
  output$optim$bestmem
}

# number of replications
R = 1000

# Monte Carlo simulation
time.test = system.time({
  MRC.simu.par = matrix(unlist(mclapply(1:R, MRC.simu, mc.cores = n.cores), use.names = F), 
                        ncol = K-1, byrow = T)
})

# time.test = system.time({
#   MRC.simu.par = t(sapply(1:R, MRC.simu))
# })

################################
# summarize simulation results #
################################

# function summarizing simulation results
simu_results = function(output, coefdgp) {
  K = length(coefdgp)
  simu_results_k = function(k) {
    Bias = output[, k] - coefdgp[k]
    meanBias = mean(Bias)
    medBias = median(Bias)
    RMSE = sqrt(mean(Bias^2))
    MAD = median(abs(Bias - medBias))
    cbind(meanBias, RMSE, medBias, MAD)
  }
  results = t(sapply(1:K, simu_results_k))
  colnames(results) = c("MEAN", "RMSE", "MED", "MAD")
  rownames(results) = paste("par", 1:K, sep = "")
  results
}

simu.table = simu_results(MRC.simu.par, coefdgp)
time.test
simu.table
write.csv(simu.table, paste("cs1_", N, ".csv", sep =""))




