
rm(list = ls())

library(MASS)
library(parallel)
library(optimx)
library(evd)

# detect the number of physical CPU cores for parallel computation
n.cores = detectCores(logical = F) 

# DGP design 5: dynamic panel data, 3 chocies, 3 regressors (including lagged y), 4 time periods
# choice set = {0,1,2}
# U1t = x1t.1 + x1t.2*b + (yt-1 = j)*r + a1 - e1t
# U2t = x2t.1 + x2t.2*b + a2 - e2t
# U0t = 0 (outside/base choice)
J = 3                             # number of choices
K = 3                             # number of regressors
TT = 4                            # number of time periods
coefdgp = c(1, 1/2)               # true parameters

LL = function(u, d1, d2){
  log((1/(1+exp(u)))^d1*(exp(u)/(1+exp(u)))^d2)
}

# sample size 
NN = c(500, 1000, 2000, 5000, 10000)

for (N in NN) {

############################
# maximum score simulation #
############################

ML.simu = function(s) {
  
  #####################################
  ## generate random sample from DGP ##
  #####################################
  
  set.seed(s)
  mu = seq(0, 0, length = (J-1))
  sigma = matrix(0.5*seq(1, 1, length = (J-1)^2), nrow = (J-1)) + diag((J-1))*0.5
  e0 = mvrnorm(n = N, mu, sigma); e10 = e0[,1]; e20 = e0[,2]
  e1 = mvrnorm(n = N, mu, sigma); e11 = e1[,1]; e21 = e1[,2]
  e2 = mvrnorm(n = N, mu, sigma); e12 = e2[,1]; e22 = e2[,2]
  e3 = mvrnorm(n = N, mu, sigma); e13 = e3[,1]; e23 = e3[,2]
  
  # EV type 1
  # e00 = rgev(N); e10 = e00 - rgev(N); e20 = e00 - rgev(N)
  # e01 = rgev(N); e11 = e01 - rgev(N); e21 = e01 - rgev(N)
  # e02 = rgev(N); e12 = e02 - rgev(N); e22 = e02 - rgev(N)
  # e03 = rgev(N); e13 = e03 - rgev(N); e23 = e03 - rgev(N)
  
  x10.1 = rbinom(N, 1, 1/2); x10.2 = rbinom(N, 1, 1/2)
  x20.1 = rbinom(N, 1, 1/2); x20.2 = rbinom(N, 1, 1/2)
  
  x11.1 = rbinom(N, 1, 1/2); x11.2 = rbinom(N, 1, 1/2)
  x21.1 = rbinom(N, 1, 1/2); x21.2 = rbinom(N, 1, 1/2)
  
  x12.1 = rbinom(N, 1, 1/2); x12.2 = rbinom(N, 1, 1/2)
  x22.1 = rbinom(N, 1, 1/2); x22.2 = rbinom(N, 1, 1/2)
  
  x13.1 = rbinom(N, 1, 1/2); x13.2 = rbinom(N, 1, 1/2)
  x23.1 = rbinom(N, 1, 1/2); x23.2 = rbinom(N, 1, 1/2)
  
  a1 = (x10.1 + x11.1 + x12.1 + x13.1 - 2)/TT/2
  a2 = (x20.1 + x21.1 + x22.1 + x23.1 - 2)/TT/2
  
  b0 = coefdgp
  
  u00 = 0; u10 = x10.1 + b0[1]*x10.2 + a1 - e10
  u20 = x20.1 + b0[1]*x20.2 + a2 - e20
  y0 = 0*(u10 < u00)*(u20 < u00) + 1*(u00 < u10)*(u20 < u10) + 2*(u00 < u20)*(u10 < u20)
  
  u01 = 0; u11 = x11.1 + b0[1]*x11.2 + b0[2]*(y0 == 1) + a1 - e11
  u21 = x21.1 + b0[1]*x21.2 + a2 - e21
  y1 = 0*(u11 < u01)*(u21 < u01) + 1*(u01 < u11)*(u21 < u11) + 2*(u01 < u21)*(u11 < u21)
  
  u02 = 0; u12 = x12.1 + b0[1]*x12.2 + b0[2]*(y1 == 1) + a1 - e12
  u22 = x22.1 + b0[1]*x22.2 + a2 - e22
  y2 = 0*(u12 < u02)*(u22 < u02) + 1*(u02 < u12)*(u22 < u12) + 2*(u02 < u22)*(u12 < u22)
  
  u03 = 0; u13 = x13.1 + b0[1]*x13.2 + b0[2]*(y2 == 1) + a1 - e13
  u23 = x23.1 + b0[1]*x23.2 + a2 - e23
  y3 = 0*(u13 < u03)*(u23 < u03) + 1*(u03 < u13)*(u23 < u13) + 2*(u03 < u23)*(u13 < u23)
  
  # match all x in t = 2 and t = 3
  K1.23 = (x12.1 == x13.1)*(x12.2 == x13.2)*(x22.1 == x23.1)*(x22.2 == x23.2)
  
  # observed choices 
  d10 = (y0 == 1); d13 = (y3 == 1)
  
  D10 = (y1 == 1)*(y2 == 0); D01 = (y1 == 0)*(y2 == 1)
  D20 = (y1 == 2)*(y2 == 0); D02 = (y1 == 0)*(y2 == 2)
  D12 = (y1 == 1)*(y2 == 2); D21 = (y1 == 2)*(y2 == 1)
  
  ######################################
  ## maximum score objective function ##
  ######################################
  
  MLobjN = function(coef) {
    
    b1 = coef[1]; b2 = coef[2]; r = coef[3]
    
    U11 = b1*x11.1 + b2*x11.2 + r*d10; U12 = b1*x12.1 + b2*x12.2 + r*d13
    U21 = b1*x21.1 + b2*x21.2; U22 = b1*x22.1 + b2*x22.2 
    
    O20 = (D20 + D02)*LL(U22-U21, D20, D02)
    O10 = (D10 + D01)*K1.23*LL(U12-U11, D10, D01)
    O12 = (D12 + D21)*K1.23*LL(U21-U22+U12-U11, D12, D21)
    
    (-1)*sum(O20 + O10 + O12)
    
  }
  
  ##################
  ## optimization ##
  ##################
  
  # optimx
  theta0 = runif(3, 0, 2); maxIt = 500; tol = 1e-8
  output = optimx(theta0, MLobjN, method = "BFGS", itnmax = maxIt, control = list(trace = T))
  c(output$p2, output$p3)
  
}

# number of replications
R = 1000

# Monte-Carlo simulation
time.test = system.time({
  ML.simu.par = matrix(unlist(mclapply(1:R, ML.simu, mc.cores = n.cores), use.names = F), 
                       ncol = K-1, byrow = T)
})


################################
# summarize simulation results #
################################

# function summarizing simulation results
simu_results = function(output, coefdgp) {
  K = length(coefdgp)
  simu_results_k = function(k) {
    Bias = output[, k] - coefdgp[k]
    meanBias = mean(Bias)
    medBias = median(Bias)
    RMSE = sqrt(mean(Bias^2))
    MAE = median(abs(Bias - medBias))
    cbind(meanBias, RMSE, medBias, MAE)
  }
  results = t(sapply(1:K, simu_results_k))
  colnames(results) = c("MEAN", "RMSE", "MED", "MAE")
  rownames(results) = paste("par", 1:K, sep = "")
  results
}

simu.table = simu_results(ML.simu.par, coefdgp)
time.test
simu.table
write.csv(simu.table, paste("logit5_", N, ".csv", sep =""))

}

